/*************************************************************************/
/**                                                                     **/
/**  MARS algorithm; CGI - Demoprogram                                  **/
/**                                                                     **/
/**  Reto Galli / 03-08-00                                              **/
/**                                                                     **/
/**  ECE 575 Project - Winter 2000                                      **/
/**                                                                     **/
/*************************************************************************/

/** This path must be changed when program is moved to another location **/
#define CGIpath "http://www.engr.orst.edu/cgi-bin/cgiwrap/~gallire/mars.cgi" 




#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "aes.h"





/*************************************************************************/
/**                                                                     **/
/**     getcgivars     routine to read CGI input variables into an      **/
/**         array of strings.                                           **/
/**                                                                     **/
/**     The x2c() and unescape_url() routines were lifted directly      **/
/**     from NCSA's sample program util.c, packaged with their HTTPD.   **/
/**                                                                     **/
/*************************************************************************/

/** Convert a two-char hex string into the char it represents **/
char x2c(char *what) {
   register char digit;

   digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
   digit *= 16;
   digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
   return(digit);
}

/** Reduce any %xx escape sequences to the characters they represent **/
void unescape_url(char *url) {
    register int i,j;

    for(i=0,j=0; url[j]; ++i,++j) {
        if((url[i] = url[j]) == '%') {
            url[i] = x2c(&url[j+1]) ;
            j+= 2 ;
        }
    }
    url[i] = '\0' ;
}


/** Read the CGI input and place all name/val pairs into list.        **/
/** Returns list containing name1, value1, name2, value2, ... , NULL  **/
char **getcgivars() {
    register int i ;
    char *request_method ;
    int content_length;
    char *cgiinput ;
    char **cgivars ;
    char **pairlist ;
    int paircount ;
    char *nvpair ;
    char *eqpos ;

    /** Depending on the request method, read all CGI input into cgiinput **/
    /** (really should produce HTML error messages, instead of exit()ing) **/
    request_method= getenv("REQUEST_METHOD") ;
    if (!strcmp(request_method, "GET") || !strcmp(request_method, "HEAD") ) {
        cgiinput= strdup(getenv("QUERY_STRING")) ;
    }
    else if (!strcmp(request_method, "POST")) {
        /* strcasecmp() is not supported in Windows-- use strcmpi() instead */
        if ( strcasecmp(getenv("CONTENT_TYPE"), "application/x-www-form-urlencoded")) {
            printf("getcgivars(): Unsupported Content-Type.\n") ;
            exit(1) ;
        }
        if ( !(content_length = atoi(getenv("CONTENT_LENGTH"))) ) {
            printf("getcgivars(): No Content-Length was sent with the POST request.\n") ;
            exit(1) ;
        }
        if ( !(cgiinput= (char *) malloc(content_length+1)) ) {
            printf("getcgivars(): Could not malloc for cgiinput.\n") ;
            exit(1) ;
        }
        if (!fread(cgiinput, content_length, 1, stdin)) {
            printf("Couldn't read CGI input from STDIN.\n") ;
            exit(1) ;
        }
        cgiinput[content_length]='\0' ;
    }
    else {
        printf("getcgivars(): unsupported REQUEST_METHOD\n") ;
        exit(1) ;
    }

    /** Change all plusses back to spaces **/
    for(i=0; cgiinput[i]; i++) if(cgiinput[i] == '+') cgiinput[i] = ' ' ;

    /** First, split on "&" to extract the name-value pairs into pairlist **/
    pairlist= (char **) malloc(256*sizeof(char **)) ;
    paircount= 0 ;
    nvpair= strtok(cgiinput, "&") ;
    while (nvpair) {
        pairlist[paircount++]= strdup(nvpair) ;
        if (!(paircount%256))
            pairlist= (char **) realloc(pairlist,(paircount+256)*sizeof(char **)) ;
        nvpair= strtok(NULL, "&") ;
    }
    pairlist[paircount]= 0 ;    /* terminate the list with NULL */

    /** Then, from the list of pairs, extract the names and values **/
    cgivars= (char **) malloc((paircount*2+1)*sizeof(char **)) ;
    for (i= 0; i<paircount; i++) {
        if (eqpos=strchr(pairlist[i], '=')) {
            *eqpos= '\0' ;
            unescape_url(cgivars[i*2+1]= strdup(eqpos+1)) ;
        } else {
            unescape_url(cgivars[i*2+1]= strdup("")) ;
        }
        unescape_url(cgivars[i*2]= strdup(pairlist[i])) ;
    }
    cgivars[paircount*2]= 0 ;   /* terminate the list with NULL */
    
    /** Free anything that needs to be freed **/
    free(cgiinput) ;
    for (i=0; pairlist[i]; i++) free(pairlist[i]) ;
    free(pairlist) ;

    /** Return the list of name-value strings **/
    return cgivars ;
    
}
/** End of getcgivars ****************************************************/






/*************************************************************************/
/**                                                                     **/
/**     generateAkey                                                    **/
/**         This routine generates a random key with the given length   **/
/**         in 32bit words                                              **/
/**                                                                     **/
/**         The key is given as an array of words and as hex number in  **/
/**         a string                                                    **/
/**                                                                     **/
/*************************************************************************/
void generateAkey(int keyLen, WORD* key, char* sKey)
{
	int ii;
   sKey[0] = 0x00;
 
	/** Initialize the pseudorandom generator with the actual system time **/
	srand((unsigned int)time((time_t*)NULL));
	
	for (ii = 0; ii < keyLen; ii++) 
   {
		/** Produce a key word **/
   	key[ii] = (2*rand()+rand()+rand())*(3*rand()+rand());
      sprintf(sKey,"%s%.8lX",sKey,key[ii]);
   }
}





/*************************************************************************/
/**                                                                     **/
/**     showKey                                                         **/
/**         Expands a key and produces a HTML output to show this key   **/
/**                                                                     **/
/*************************************************************************/
void showKey(int keyLen, WORD* key, int genKey)
{
   int ii;
   WORD e[EKEY_WORDS];
   char s[10];

   (void)mars_setup(keyLen,key,e);

   if (genKey)
      strcpy(s,"generated");
   else
      strcpy(s,"provided");    
	 
	printf("<i>Result of your last key request:</i><br>");
   printf("The %s %d bit encryption key:</font><br>",s,32*keyLen);
   printf("<tt>%.8lX %.8lX %.8lX %.8lX",key[0], key[1], key[2], key[3]);
   for (ii = 4; ii < keyLen; ii++) printf(" %.8lX",key[ii]);
   printf("</tt><br><br>\n\n");
	printf("<font face=\"Arial,Helvetica\">\n");
   printf("Leads to the expanded key:</font><tt>");
   for (ii = 0; ii < 40; ii++) 
   {
      if (!(ii%10)) printf("<br>");
      printf(" %.8lX",e[ii]);
   }
   printf("</tt><font face=\"Arial,Helvetica\"><br><br>\n\n");

}





/*************************************************************************/
/**                                                                     **/
/**     doSingleBlock                                                   **/
/**         Encrypts or decrypts a single MARS block and produces a     **/
/**         HTML output to show the plaintext and the ciphertext block. **/
/**                                                                     **/
/*************************************************************************/
void doSingleBlock(WORD* pt_in, int keyLen, WORD* key, int EncDcr, char* sBlock)
{
    WORD pt_out[NUM_DATA];
    WORD e[EKEY_WORDS];
    int ii;

    (void)mars_setup(keyLen,key,e);

    printf("<i>Result of your last request:</i><br>\n");

    if (EncDcr == DIR_ENCRYPT)
    {
       mars_encrypt(pt_in,pt_out,e);
       printf("Action: encrypt<br></font>\n");
       printf("<tt>plaintext  input: %.8lX %.8lX %.8lX %.8lX<br>\n",pt_in[0], pt_in[1], pt_in[2], pt_in[3]);
       printf("encrypted output: %.8lX %.8lX %.8lX %.8lX</tt><br>\n",pt_out[0], pt_out[1], pt_out[2], pt_out[3]);
    }
    else
    {
       mars_decrypt(pt_in,pt_out,e);
       printf("Action: decrypt<br></font>");
       printf("<tt>encrypted  input: %.8lX %.8lX %.8lX %.8lX<br>\n",pt_in[0], pt_in[1], pt_in[2], pt_in[3]);
       printf("plaintext output: %.8lX %.8lX %.8lX %.8lX</tt><br>\n",pt_out[0], pt_out[1], pt_out[2], pt_out[3]);
    }
 	 printf("<br><font face=\"Arial,Helvetica\">\n");

    sprintf(sBlock,"%.8lX%.8lX%.8lX%.8lX",pt_out[0], pt_out[1], pt_out[2], pt_out[3]);
   
}





/*************************************************************************/
/**                                                                     **/
/**     doChainEncrypt                                                  **/
/**         Encrypts or decrypts a byte stream with MARS using the      **/
/**         given chainmode (ECB or CBC/IV)                             **/
/**                                                                     **/
/*************************************************************************/
doChainEncrypt(BYTE* inBuf, BYTE* outBuf, int bufLen, WORD* e, int Chainmode, WORD* IV)
{
   WORD pt_in[NUM_DATA];
   WORD pt_out[NUM_DATA];
   int i1;
   int i2;

	for (i1 = 0; i1 < (bufLen/16); i1++)
   {
   	for (i2 = 0; i2 < NUM_DATA; i2++)
			pt_in[i2] = inBuf[16*i1+4*i2+3] + 
						  (inBuf[16*i1+4*i2+2] << 8) + 
						  (inBuf[16*i1+4*i2+1] << 16) + 
						  (inBuf[16*i1+4*i2+0] << 24);
		
		if (Chainmode == MODE_CBC)
		{
			if (i1 == 0)
		   	for (i2 = 0; i2 < NUM_DATA; i2++) pt_in[i2] = pt_in[i2] ^ IV[i2];
			else
		   	for (i2 = 0; i2 < NUM_DATA; i2++) pt_in[i2] = pt_in[i2] ^ pt_out[i2];
		}

      mars_encrypt(pt_in,pt_out,e);

   	for (i2 = 0; i2 < NUM_DATA; i2++)
		{
			outBuf[16*i1+4*i2+0] = (BYTE) ((pt_out[i2] & 0xFF000000) >> 24);
         outBuf[16*i1+4*i2+1] = (BYTE) ((pt_out[i2] & 0x00FF0000) >> 16);
         outBuf[16*i1+4*i2+2] = (BYTE) ((pt_out[i2] & 0x0000FF00) >> 8);
         outBuf[16*i1+4*i2+3] = (BYTE) ((pt_out[i2] & 0x000000FF));
		}

	}
}





/*************************************************************************/
/**                                                                     **/
/**     doChainDecrypt                                                  **/
/**         Encrypts or decrypts a byte stream with MARS using the      **/
/**         given chainmode (ECB or CBC/IV)                             **/
/**                                                                     **/
/*************************************************************************/
doChainDecrypt(BYTE* inBuf, BYTE* outBuf, int bufLen, WORD* e, int Chainmode, WORD* IV)
{
   WORD pt_in[NUM_DATA];
   WORD pt_out[NUM_DATA];
	WORD CBC[NUM_DATA];
   int i1;
   int i2;

	for (i1 = 0; i1 < (bufLen/16); i1++)
   {
   	for (i2 = 0; i2 < NUM_DATA; i2++)
			pt_in[i2] = inBuf[16*i1+4*i2+3] + 
						  (inBuf[16*i1+4*i2+2] << 8) + 
						  (inBuf[16*i1+4*i2+1] << 16) + 
						  (inBuf[16*i1+4*i2+0] << 24);

      mars_decrypt(pt_in,pt_out,e);

		if (Chainmode == MODE_CBC)
		{
			if (i1 == 0)
		   	for (i2 = 0; i2 < NUM_DATA; i2++) pt_out[i2] = pt_out[i2] ^ IV[i2];
			else
			  	for (i2 = 0; i2 < NUM_DATA; i2++) pt_out[i2] = pt_out[i2] ^ CBC[i2];
			
	   	for (i2 = 0; i2 < NUM_DATA; i2++) CBC[i2] = pt_in[i2];
			
		}

   	for (i2 = 0; i2 < NUM_DATA; i2++)
		{
			outBuf[16*i1+4*i2+0] = (BYTE) ((pt_out[i2] & 0xFF000000) >> 24);
         outBuf[16*i1+4*i2+1] = (BYTE) ((pt_out[i2] & 0x00FF0000) >> 16);
         outBuf[16*i1+4*i2+2] = (BYTE) ((pt_out[i2] & 0x0000FF00) >> 8);
         outBuf[16*i1+4*i2+3] = (BYTE) ((pt_out[i2] & 0x000000FF));
		}

	}
}






/*************************************************************************/
/**                                                                     **/
/**     showChainCrypt                                                  **/
/**         Produces a HTML output to show what doChainEncypt or        **/
/**         doChainDecrypt did.                                         **/
/**                                                                     **/
/*************************************************************************/
int showChainCrypt(int keyLen, WORD* Key, int Chainmode, int dirMode, WORD* wIV, char* Input, char** Output)
{
   BYTE* inAsBuf;
   BYTE* inHxBuf;
   BYTE* outAsBuf;
   BYTE* outHxBuf;
   int iTextLen;
	int iAsBufLen;
	int iHxBufLen;
   char s[3];
   int ii;
   WORD e[EKEY_WORDS];
	char sHTMLOut[65];

   s[2] = 0x00;
   iTextLen = strlen(Input);
 
	if (dirMode == DIR_DECRYPT) 
	{
		iHxBufLen = iTextLen;	
		iTextLen = iTextLen/2;
		iAsBufLen =  iTextLen;
		if (iTextLen%16 != 0)
		{
			printf("Invalid cipher text<br>");
			Output[0] = Input;
			return -1;
		} 
	}
	else
	{
	   iAsBufLen = iTextLen + (16 - (iTextLen%16));
   	iHxBufLen = 2*iAsBufLen;
	}
  	

	inAsBuf  = (BYTE*) malloc(iAsBufLen+1);
	outAsBuf = (BYTE*) malloc(iAsBufLen+1);
   inHxBuf  = (BYTE*) malloc(iHxBufLen+1);
   outHxBuf = (BYTE*) malloc(iHxBufLen+1);


	if (dirMode == DIR_ENCRYPT)
   {  
		strncpy((char*)inAsBuf,Input,iTextLen);
      for (ii = iTextLen; ii <= iAsBufLen; ii++) inAsBuf[ii] = 0x00;
      inHxBuf[0] = 0x00;
      for (ii = 0; ii < iAsBufLen; ii++) sprintf((char*)inHxBuf,"%s%.2X",(char*)inHxBuf,inAsBuf[ii]);
	}
	else
   {
		strncpy((char*)inHxBuf,Input,iHxBufLen);
      inHxBuf[iHxBufLen] = 0x00;
      for (ii = 0; ii < iAsBufLen; ii++) 
      {
		   strncpy(s, (char*)&inHxBuf[ii*2], 2);
         inAsBuf[ii] = (BYTE) strtol(s, NULL, 16);
      }
      inAsBuf[iAsBufLen] = 0x00;
   }

   (void)mars_setup(keyLen,Key,e);

	printf("<i>Result of your last request:</i><br><br>");
 
   if (dirMode == DIR_ENCRYPT)
   {
      printf("Your input text: <br></font><tt>%s <br></tt>\n<font face=\"Arial,Helvetica\"><br>\n",(char*)inAsBuf);
      printf("is equal to the bytestream: <br></font><tt>\n"); 
		for (ii = 0; ii<strlen((char*) inHxBuf);ii += 64)
		{
			strncpy(sHTMLOut,(char*)&inHxBuf[ii],64); sHTMLOut[64] = 0x00;
	      printf("%s<br>\n",sHTMLOut); 
		}
		printf("</tt><font face=\"Arial,Helvetica\"><br>");	

      doChainEncrypt(inAsBuf, outAsBuf, iAsBufLen, e, Chainmode, wIV);

   	outHxBuf[0] = 0x00;
      for (ii=0; ii<iAsBufLen+16; ii++) sprintf((char*)outHxBuf,"%s%.2X",(char*)outHxBuf,outAsBuf[ii]);
      outHxBuf[iHxBufLen] = 0x00;

		Output[0] = (char*)outHxBuf; 
       
      printf("the encrypted bytestream is: <br></font><tt>\n"); 
		for (ii = 0; ii<strlen((char*) outHxBuf);ii += 64)
		{
			strncpy(sHTMLOut,(char*)&outHxBuf[ii],64); sHTMLOut[64] = 0x00;
	      printf("%s<br>\n",sHTMLOut); 
		}
		printf("</tt><font face=\"Arial,Helvetica\"><br>");	
   }
	else
   {
	   printf("Your cipher bytestream: <br></font><tt>\n");
		for (ii = 0; ii<strlen((char*) inHxBuf);ii += 64)
		{
			strncpy(sHTMLOut,(char*)&inHxBuf[ii],64); sHTMLOut[64] = 0x00;
	      printf("%s<br>\n",sHTMLOut); 
		}
		printf("</tt><font face=\"Arial,Helvetica\"><br>");	

      doChainDecrypt(inAsBuf, outAsBuf, iAsBufLen, e, Chainmode, wIV);

      for (ii=0; ii<iAsBufLen; ii++) sprintf((char*)outHxBuf,"%s%.2X",(char*)outHxBuf,outAsBuf[ii]);
      outHxBuf[iHxBufLen] = 0x00;
      
		printf("leads to the plaintext bytestream: <br></font><tt>\n");
		for (ii = 0; ii<strlen((char*) outHxBuf);ii += 64)
		{
			strncpy(sHTMLOut,(char*)&outHxBuf[ii],64); sHTMLOut[64] = 0x00;
	      printf("%s<br>\n",sHTMLOut); 
		}
		printf("</tt><font face=\"Arial,Helvetica\"><br>");	

      printf("Which is equal with the text: <br></font><tt>%s <br></tt>\n<font face=\"Arial,Helvetica\"><br>\n",(char*)outAsBuf); 
      Output[0] = (char*) outAsBuf;
      
   }
	
	if (dirMode == DIR_ENCRYPT)
	   free(outAsBuf);
   else
		free(outHxBuf);

	free(inHxBuf);
	free(inAsBuf);

	return 0;
}




/*************************************************************************/
/**                                                                     **/
/**     interpretInput                                                  **/
/**         Converts a long hex number from a string into an array of   **/
/**         words                                                       **/
/**                                                                     **/
/*************************************************************************/
void interpretInput(char* inputLine, WORD* buf, int bufLen)
{
	int ii;
	char s[9];
   
   s[8] = 0x00;
   for (ii = 0; ii < bufLen; ii++)
   {
		strncpy(s, &inputLine[ii*8], 8);
      buf[ii] = strtoul(s, NULL, 16);
   }
}




/*************************************************************************/
/**                                                                     **/
/**     main                                                            **/
/**         Interprets the input variables and produces the HTML output **/
/**                                                                     **/
/*************************************************************************/
int main() 
{
	char **cgivars ;

   WORD  wBlock[NUM_DATA];
   WORD  wKey[EKEY_WORDS];
   WORD  wIV[4];
   char  sKey[EKEY_WORDS*8+1];
	char  sIV[NUM_DATA*8+1];
   char  sBlock[NUM_DATA*8+1];
   char* sInputText;
	char* sOutputText;
	 
	int   iKeyLen;
   int   fSbEncDcr;
   int   fChEncDcr;
   int   fChainmode;

   int   iKeyIndex, iBlockIndex, iIvIndex;
   int   fSelIV;
   int   fGenKey;
   int   fSingleBlock;
   int   fChainCrypt;
   int   iError;
	int	fInit;

   int   ii ;
   char* s;
   char  sWords[3];
    

	/** get the CGI variables into a list of strings **/
   cgivars= getcgivars();      


	/** Interpret the input values **/
	iError = 0;

   iKeyLen = 4;
   fGenKey = FALSE;
   fSelIV = -1;
   fSbEncDcr = DIR_DECRYPT;
   fChEncDcr = DIR_DECRYPT;
   sInputText = NULL;
	fChainmode = MODE_ECB;
   
	iKeyIndex = -1;
   iBlockIndex = -1;
   iIvIndex = -1;
   
	fSingleBlock = TRUE;
   fChainCrypt = TRUE;

	fInit = FALSE;

   for (ii=0; cgivars[ii]; ii+= 2)
   {
		if (!strncmp(cgivars[ii],"KeyLen",6))
      {
         s = strchr(cgivars[ii+1],'(');
         sWords[0] = s[1]; sWords[1] = s[2]; sWords[2] = 0x00;
         iKeyLen = atoi(sWords);
      }
      if (!strncmp(cgivars[ii],"Key",3)) iKeyIndex = ii+1;
      if (!strncmp(cgivars[ii],"Block",5)) iBlockIndex = ii+1;
      if (!strncmp(cgivars[ii],"IVBlockVal",10)) iIvIndex = ii+1;

      if (!strncmp(cgivars[ii],"Text",4)) sInputText = cgivars[ii+1];
      if (!strncmp(cgivars[ii],"GenKey",6)) if (!strncmp(cgivars[ii+1],"GenKey",6)) fGenKey = TRUE;
      if (!strncmp(cgivars[ii],"sbEncDcr",8)) if (!strncmp(cgivars[ii+1],"encrypt",6)) fSbEncDcr = DIR_ENCRYPT;
      if (!strncmp(cgivars[ii],"chEncDcr",8)) if (!strncmp(cgivars[ii+1],"encrypt",6)) fChEncDcr = DIR_ENCRYPT;
      if (!strncmp(cgivars[ii],"ivBlockSel",9))
      { 
         if (!strncmp(cgivars[ii+1],"ThisIV",5)) fSelIV = 0;
         if (!strncmp(cgivars[ii+1],"GenIV",5)) fSelIV = 1;
         if (!strncmp(cgivars[ii+1],"ZeroIV",5)) fSelIV = 2;
      }
      if (!strncmp(cgivars[ii],"Chainmode",9))
      { 
         if (!strncmp(cgivars[ii+1],"ECB",3)) fChainmode = MODE_ECB;
         if (!strncmp(cgivars[ii+1],"CBC",3)) fChainmode = MODE_CBC;
      }
      if (!strncmp(cgivars[ii],"Init",10)) fInit = TRUE;
   }


   /* Print the CGI response header, required for all HTML output. */
   printf("Content-type: text/html\n\n") ; 
    
   /* Print out the title of the HTML response page. */
   printf("<html>\n") ;
   printf("<head><title>MARS encryption algorithm</title></head>\n") ;
   printf("<body>\n") ;
	printf("<FORM action=\""CGIpath"\" method=post>\n");
	printf("<font face=\"Arial,Helvetica\">\n");
   printf("<h1>MARS encryption algorithm demo program</h1></font>\n"); 
	printf("<HR WIDTH=\"100%%\">\n");

	if (!fInit)
	{

		/** Make some tests about the input values **/
	
   	if (!((iKeyLen >=4) && (iKeyLen <=14)))
      	iError = 1001;
		   else
      	if (!(fGenKey) && (iKeyLen*8 != strlen(cgivars[iKeyIndex])))
         	iError = 1002;
	      else
		   {
      	   interpretInput(cgivars[iKeyIndex],wKey,iKeyLen);
         	strcpy(sKey,cgivars[iKeyIndex]);
	      }

	   if ((iBlockIndex == -1) || (32 != strlen(cgivars[iBlockIndex])))
   	   fSingleBlock = FALSE;
	   else
      	interpretInput(cgivars[iBlockIndex],wBlock,NUM_DATA);

		if (fGenKey) generateAkey(iKeyLen,wKey,sKey);        

		if (sInputText)
	 	{
    		if (fSelIV == 0)
   	   	if (32 == strlen(cgivars[iIvIndex]))
				{
   	   	   interpretInput(cgivars[iIvIndex],wIV,NUM_DATA);
	   	      strcpy(sIV,cgivars[iIvIndex]);
				}
		   	else
				{
					if (fChainmode == MODE_CBC)
					{
	   		      iError = 2001;
						fChainCrypt = FALSE;
					}
					else
					{
		   		   generateAkey(NUM_DATA,wIV,sIV);
					}
				}
		   else if (fSelIV == 1)
   		   generateAkey(NUM_DATA,wIV,sIV);
	   	else if (fSelIV == 2)
			{
 			   wIV[0] = wIV[1] = wIV[2] = wIV[3] = 0;
				strcpy(sIV,"00000000000000000000000000000000");
			}
		   else
			{
   		   iError = 2002;
				fChainCrypt = FALSE;					
			}
	   }
		else
		{
			iError = 2000;
			fChainCrypt = FALSE;				
		}
	}
	else
	{
		/** Put in default values **/
		fSingleBlock = FALSE;
		fChainCrypt =  FALSE;
		iKeyLen = 4;
		strcpy(sKey,"A3982F23E1A4BB29FE3122598C2A31FB");
		strcpy(sIV, "9387A2EF546534F36B473269A97CC432");
		strcpy(sBlock, "153A3B7B533F4987A67547E6EFB76A81");
		sOutputText = (char*) malloc(100);
		strcpy(sOutputText,"This text will be encrypted using MARS algorithm with the above key and the chaining mode provided.");		
	}

/*
   printf("Debug output:\n") ;
   printf("<ul>\n") ;
   for (ii=0; cgivars[ii]; ii+= 2)
       printf("<li>[%s] = [%s]\n", cgivars[ii], cgivars[ii+1]) ;
   printf("</ul>\n") ;
                                                                  */


	/** Show the HTML response **/
	printf("<p>\n");
	printf("<font face=\"Arial,Helvetica\">\n");
	printf("<B><I>Encryption/decryption key</B></I><br><br>\n");

	/** Key **/
   if ((iError != 1001)&&(iError != 1002)&&(!fInit))
   {
      showKey(iKeyLen,wKey,fGenKey);
   }
   else
   {
      fSingleBlock = FALSE;
		fChainCrypt = FALSE;
      if (!fInit) printf("Your last request contained invalid key information!!!<br><br>");
   }

   printf("<i>Define a new key: </i><br>");	
   printf("Key length: \n");
	printf("<select name=\"KeyLen\">\n");
	printf("   <option> %3d bit (%2d words)\n",iKeyLen*32,iKeyLen);
	for (ii = 4; ii <= 14; ii++) if (ii != iKeyLen) printf("   <option> %3d bit (%2d words)\n",ii*32,ii);
	printf("</select> \n");
	printf("key (in HEX):  \n");
	printf("<tt>\n");
	printf("   <input size=32 maxlength=312 name=\"Key\" value=\"%s\">\n",sKey);
	printf("</tt>\n");
	printf("<br>\n");
	printf("<I>Note: The key you provide must be exactly in length (One word is 8 HEX-digits).</I><br><br>\n");
	printf("<B><I>or</I></B>\n");
	printf("<input type=checkbox name=\"GenKey\" value=\"GenKey\">\n");
	printf("let the program generate a key for you with the given length\n");
	printf("</font></p>\n");


	/** Single Block encryption/decryption **/
	printf("<p><font face=\"Arial,Helvetica\">\n");
	printf("<HR WIDTH=\"100%%\"><br>\n");
	printf("<B><I>Low-level test of the algorithm: </I></B><I>(Single block encryption/decryption)</I> <br><br>\n");

   if (fSingleBlock)
      doSingleBlock(wBlock,iKeyLen,wKey,fSbEncDcr,sBlock);
   else
      if (!fInit) 
		{
			printf("Your last request contained no valid data for encryption/decryption of a single block.<br><br>");
			strcpy(sBlock, "153A3B7B533F4987A67547E6EFB76A81");
		}


	printf("New block: \n");
	printf("<tt>\n");
	printf("<input size=32 maxlength=32 name=\"Block\" value=\"%s\">\n",sBlock);
	printf("</tt>\n");
	if (fSbEncDcr == DIR_ENCRYPT)
	{
		printf("<input type=radio name=\"sbEncDcr\" value=\"encrypt\"> encrypt  \n");
		printf("<input type=radio checked name=\"sbEncDcr\" value=\"decrypt\"> decrypt  \n");
	}
	else
	{
		printf("<input type=radio checked name=\"sbEncDcr\" value=\"encrypt\"> encrypt  \n");
		printf("<input type=radio name=\"sbEncDcr\" value=\"decrypt\"> decrypt  \n");
	}
	printf("<br>\n");
	printf("<I>Note: The block you provide must be exactly 128 bit (4 words -> 32 HEX digits)</I>\n");
	printf("</font></p>\n");



	/** Chain encryption / decryption **/
	printf("<p><font face=\"Arial,Helvetica\">\n");
	printf("<HR WIDTH=\"100%%\"><br>\n");
	printf("<b><i>High-level test of the algorithm</i></b><br>\n");
	printf("With the option <i>encrypt</i> you can provide a text that will be encrypted with the above key using one of \n");
	printf("of the chaining modes: ECB (Electronic Codebook), CBC (Cipher Block Chaining). As result you will get a stream \n");
	printf("of bytes (in hex).<BR> \n");
	printf("With the option <i>decrypt</i> you can provide a stream of bytes (in hex), which will be decrypted with the\n");
	printf("above key, using one of the chaining modes.<BR><BR> \n\n");

	if (fChainCrypt) 
	{
		sOutputText = NULL;
		showChainCrypt(iKeyLen,wKey,fChainmode,fChEncDcr,wIV,sInputText,&sOutputText);
	}
	else 
	{
		if (!fInit)
		{
			sOutputText =  sInputText;
			printf("Your last request contained no valid data for encryption/decryption of a text.<br><br>");
		}
	}

	printf("New text or byte stream:<BR> \n");
	printf("<textarea wrap=virtual cols=40 rows=10 name=\"Text\">\n");
	if (sOutputText)	printf("%s",sOutputText);
	printf("</textarea>\n");
	if (fChEncDcr == DIR_ENCRYPT)
	{
		printf("<input type=radio name=\"chEncDcr\" value=\"encrypt\"> encrypt\n");  
		printf("<input type=radio checked name=\"chEncDcr\" value=\"decrypt\"> decrypt\n");  
	}
	else
	{
		printf("<input type=radio checked name=\"chEncDcr\" value=\"encrypt\"> encrypt\n");  
		printf("<input type=radio name=\"chEncDcr\" value=\"decrypt\"> decrypt\n");  
	}
	printf("<br><br>\n");
	printf("Mode:\n"); 
	printf("<select name=\"Chainmode\">\n");
	if (fChainmode == MODE_ECB)
	{
		printf("   <option>ECB\n");
		printf("   <option>CBC\n");
	}
	else
	{
		printf("   <option>CBC\n");
		printf("   <option>ECB\n");
	}
	printf("</select>\n");
	printf("IV-Block: \n");
	printf("<tt>\n");
	printf("<input size=32 maxlength=32 name=\"IVBlockVal\" value=\"%s\">\n",sIV);
	printf("</tt><br>\n");
	printf("<input type=radio checked name=\"ivBlockSel\" value=\"ThisIV\"> take this IV  \n");
	printf("<input type=radio name=\"ivBlockSel\" value=\"GenIV\"> generate random IV\n");
	printf("<input type=radio name=\"ivBlockSel\" value=\"ZeroIV\"> take zero as IV\n");
	printf("<br><i>Note: The IV you provide must be exactly 128 bit (4 words -> 32 HEX digits)</i>\n");
	printf("</font></p>\n");
	if (sOutputText) free(sOutputText);

	
	/** End of the page **/
	printf("<p><font face=\"Arial,Helvetica\">\n");
	printf("<HR WIDTH=\"100%%\">\n");
	printf("<input type=submit value=\"encrypt/decrypt\">   <input type=reset value=\"undo changes\">\n");
	printf("    <a href=\"%s?Init=Init\">Start over with default values</A>",CGIpath);
	printf("<HR WIDTH=\"100%%\">\n");

	printf("</font></p>\n");
	printf("</FORM>\n");
	
	
	printf("<p><font face=\"Arial,Helvetica\">\n");
	printf("<TABLE COLS=2 WIDTH=\"100%%\" >\n");
	printf("<TR VALIGN=TOP>\n");
	printf("   <TD><A HREF=\"mailto:reto@orst.edu\">Reto Galli&lt;reto@orst.edu></A></TD>\n");
	printf("   <TD><DIV ALIGN=right>03/06/2000</DIV></TD>\n");
	printf("</TR>\n");
	printf("</TABLE>\n");
	printf("</font></p>\n");

   printf("</body>\n") ;
   printf("</html>\n") ;

   /** Free anything that needs to be freed **/
   for (ii=0; cgivars[ii]; ii++) free(cgivars[ii]) ;
   free(cgivars) ;

   exit(0) ;
}

